#!/bin/bash

set -e -o pipefail

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintFlagsFinal"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/tmp/error_java_identity_%p.log"
JAVA_OPTS+=" -Xlog:gc*:/tmp/migration.txt:time,level,tags:filesize=50M,filecount=1"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null; then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

JAVA_OPTS+=" -Xms${JVM_XMS:-768m} -Xmx${JVM_XMX:-1024m}"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

JAVA_OPTS+=" -Xss384k"
JAVA_OPTS+=" -XX:MaxJavaStackTraceDepth=0"
JAVA_OPTS+=" -XX:SurvivorRatio=8"
JAVA_OPTS+=" -XX:TargetSurvivorRatio=90"
JAVA_OPTS+=" -XX:MaxTenuringThreshold=15"
JAVA_OPTS+=" -XX:ParallelGCThreads=4"
JAVA_OPTS+=" -XX:+OptimizeStringConcat"

HEAP_DUMP_PATH="/heapDump/$( date '+%Y-%m-%d_%H-%M-%S' )_heap_dump.hprof"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"

if [[ -z "${MIGRATION_PORT}" || -z "${MIGRATION_PROTOCOL}" ]]; then
	if [[ "${MIGRATION_HOST}" =~ ".amazonaws.com" ]]; then
		MIGRATION_PORT="443"
		MIGRATION_PROTOCOL="https"
	else
		MIGRATION_PORT="9200"
		MIGRATION_PROTOCOL="http"
	fi
fi

if [ "$MIGRATION_MODE" == "source" ]; then
    JAVA_OPTS+=" -DauditLog.migration.destination.port=8081"
    JAVA_OPTS+=" -DauditLog.coordination.port=8082"
    JAVA_OPTS+=" -DauditLog.migration.source.enabled=true"
    JAVA_OPTS+=" -DauditLog.migration.source.start=true"
    JAVA_OPTS+=" -DauditLog.migration.destination.enabled=false"

    if [ -n "$MIGRATION_PROTOCOL" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.es.protocol=$MIGRATION_PROTOCOL"
    fi

    if [ -n "$MIGRATION_HOST" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.es.host=$MIGRATION_HOST"
    fi

    if [ -n "$MIGRATION_PORT" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.es.port=$MIGRATION_PORT"
    fi

    if [ -n "$MIGRATION_USER" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.es.user=$MIGRATION_USER"
    fi

    if [ -n "$MIGRATION_PASS" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.es.pass=$MIGRATION_PASS"
    fi

    if [ -n "$MIGRATION_ENTITY" ]; then
        JAVA_OPTS+=" -DauditLog.migration.entity=$MIGRATION_ENTITY"
    fi

    if [ -n "$MIGRATION_AUDIT_LOG_SOURCE_INDEX" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.audit.indexName=$MIGRATION_AUDIT_LOG_SOURCE_INDEX"
    fi

    if [ -n "$MIGRATION_CONNECTOR_SOURCE_INDEX" ]; then
        JAVA_OPTS+=" -DauditLog.migration.source.connector.indexName=$MIGRATION_CONNECTOR_SOURCE_INDEX"
    fi
elif [ "$MIGRATION_MODE" == "destination" ]; then
    JAVA_OPTS+=" -DauditLog.migration.destination.enabled=true"
    JAVA_OPTS+=" -DauditLog.migration.source.enabled=false"
    JAVA_OPTS+=" -DauditLog.migration.source.start=false"

    if [ -n "$MIGRATION_PROTOCOL" ]; then
        JAVA_OPTS+=" -DauditLog.migration.destination.es.protocol=$MIGRATION_PROTOCOL"
    fi

    if [ -n "$MIGRATION_HOST" ]; then
        JAVA_OPTS+=" -DauditLog.migration.destination.es.host=$MIGRATION_HOST"
    fi

    if [ -n "$MIGRATION_PORT" ]; then
        JAVA_OPTS+=" -DauditLog.migration.destination.es.port=$MIGRATION_PORT"
    fi

    if [ -n "$MIGRATION_USER" ]; then
        JAVA_OPTS+=" -DauditLog.migration.destination.es.user=$MIGRATION_USER"
    fi

    if [ -n "$MIGRATION_PASS" ]; then
        JAVA_OPTS+=" -DauditLog.migration.destination.es.pass=$MIGRATION_PASS"
    fi
elif [ "$MIGRATION_MODE" == "coordination" ]; then
    JAVA_OPTS+=" -DauditLog.migration.destination.enabled=false"
    JAVA_OPTS+=" -DauditLog.migration.source.enabled=false"
    JAVA_OPTS+=" -DauditLog.migration.source.start=false"
    JAVA_OPTS+=" -DauditLog.coordination.enabled=true"

    if [ -n "$MIGRATION_START_DATE" ]; then
        JAVA_OPTS+=" -DauditLog.coordination.startYearAndMonth=$MIGRATION_START_DATE"
    fi
fi

if [ -n "COORDINATION_URL" ]; then
    JAVA_OPTS+=" -DauditLog.coordination.url=$COORDINATION_URL"
fi

if [ -n "$ENV_NAME" ]; then
    JAVA_OPTS+=" -DauditLog.migration.source.envName=$ENV_NAME"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

echo "JAVA_OPTS: $JAVA_OPTS"

JAR_NAME="migration-service.jar"
if [ "$MIGRATION_IMPL" == "os25" ]; then
    JAR_NAME="migration-service-os25.jar"
elif [ "$MIGRATION_IMPL" == "es56" ]; then
    JAR_NAME="migration-service-es56.jar"
elif [ "$MIGRATION_IMPL" == "es68" ]; then
    JAR_NAME="migration-service-es68.jar"
fi

echo -e '\n\n==============\n\n\n'

cd "$MIGRATION_ROOT"
java \
    $JAVA_OPTS \
    -jar $JAR_NAME \
    "$@" &
PID=$!

EXITCODE=0
wait "$PID" || EXITCODE=$?
echo "Java process exited with $EXITCODE"
exit "$EXITCODE"
